@@ -1,3 +1,4 @@
+# vim: set fileencoding=utf-8:
 # GNU Solfege - free ear training software
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
 #
@@ -42,6 +43,9 @@ def mk_abs(filename, save_location):
         return os.path.join(save_location, filename)
     return filename
 
+def escape(s):
+    return s.replace("\\", "\\\\").replace("'", "\\'").replace('"', '\\"')
+
 def get_front_pages_list(debug):
     files = []
     def is_frontpage(fn):
@@ -64,8 +68,9 @@ def get_front_pages_list(debug):
     # The next line is for pre 3.15 compat. We used to place learning
     # trees there.
     files.extend(add_subdir(os.path.join(filesystem.app_data(), u"learningtrees")))
+    # This is the recommended place to save front page files
     files.extend([
-        fn for fn in glob.glob(os.path.join(filesystem.app_data(), u"exercises", "*", "*"))
+        fn for fn in glob.glob(os.path.join(filesystem.user_data(), u"exercises", "*", "*"))
         if is_frontpage(fn)])
     if not debug:
         try:
@@ -91,7 +96,7 @@ class _TreeCommon(list):
             if isinstance(child, _TreeCommon):
                 child.dump(stream, level + 1)
             else:
-                print >> stream, "%su'%s'," % (" " * (level + 1), child)
+                print >> stream, "%su'%s'," % (" " * (level + 1), escape(child))
     def iterate_filenames(self):
         """
         Yield the filenames of each lesson that has been added to this
@@ -161,12 +166,12 @@ class _NamedTreeCommon(_TreeCommon):
     def __init__(self, name=u'', listitems=[]):
         assert isinstance(name, unicode)
         _TreeCommon.__init__(self, listitems)
-        self.m_name = name 
+        self.m_name = name
     def dump(self, stream, level=0):
-        print >> stream, "%s%s(_(u'%s'), [" % (" " * level, self.__class__.__name__, self.m_name)
+        print >> stream, "%s%s(_(u'%s'), [" % (" " * level, self.__class__.__name__, escape(self.m_name))
         self.dump_children(stream, level)
         print >> stream, "%s ])," % (" " * level)
-        
+
 class LinkList(_NamedTreeCommon):
     """
     A list of links leading to exercises or new pages.
@@ -294,7 +299,7 @@ def may_be_frontpage(filename):
             return False
         return False
 
-    
+
 def parse_tree(s, C_locale=False):
     """
     This function can load both new (3.15.2) and older learning trees.
@@ -309,7 +314,14 @@ def parse_tree(s, C_locale=False):
             return s if isinstance(s, unicode) else unicode(s)
         ifunc = ifu
     else:
-        ifunc = _
+        # The headings and links to exercises or pages should not have
+        # accels (defined in gtk+ by underlines "_") or ellipsis "…" that
+        # are used in menu items and command button labels. So we remove
+        # them, if found, in the string. This way we can reuse strings
+        # with accels that are translated, saving translator work.
+        def _s(s):
+            return _(s).replace("_", "").replace(u"…", "")
+        ifunc = _s
     namespace = {
         'FileHeader': FileHeader,
         'Page': Page,
