@@ -469,43 +469,18 @@ class MpdParsable(MusicBaseClass):
             # added music code, not in the m_musicdata code.
             return
         exception.m_obj_lineno = exception.m_lineno - 1
-    def get_rhythmstaff(self, lessonfile_ref):
+    def get_score(self, lessonfile_ref, as_name=None):
         """
-        Create and return a RhythmStaff showing the rhythm of the music
-        object.
+        Return a elems.Score object of the music object. Report the variable
+        named by the as_name variable as the music object variable with the
+        bug if a MpdException is raised.
         """
-        score = elems.Score()
-        staff = score.add_staff(staff_class=elems.RhythmStaff)
-        timesig = None
-        # the current note/rest-duration
-        duration = Duration(4, 0)
-        tie_is_in_the_air = False
-        for toc_type, toc in mpd.parser.Lexer(
-                self.get_mpd_music_string(lessonfile_ref)):
-            if toc_type in (mpd.parser.Lexer.NOTE, mpd.parser.Lexer.REST):
-                if timesig is None:
-                    # very first note and no timesig parsed
-                    score.add_bar(elems.TimeSignature(4, 4))
-                    timesig = elems.TimeSignature(4, 4)
-                if score.voice11.is_bar_full():
-                    score.add_bar(timesig)
-                if toc.m_duration:
-                    duration = toc.m_duration
-                score.voice11.append(elems.Note(toc.m_pitch, duration))
-                if tie_is_in_the_air:
-                    score.voice11[-2]['elem'][0].m_tieinfo = 'start'
-                    score.voice11[-1]['elem'][0].m_tieinfo = 'end'
-                    tie_is_in_the_air = False
-            elif toc_type == mpd.parser.Lexer.TIME:
-                timesig = elems.TimeSignature(toc.m_num, toc.m_den)
-                score.add_bar(timesig)
-            elif toc_type == u'~':
-                tie_is_in_the_air = True
-            elif toc_type == u'|':
-                if not score.voice11.is_bar_full():
-                    logging.warning(u"bar check in question failed.")
-        return score
-
+        try:
+            return mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
+        except mpd.MpdException, e:
+            if as_name:
+                e.m_mpd_varname = as_name
+            raise
 
 
 class MpdDisplayable(MpdParsable):
@@ -859,24 +834,29 @@ class PercBaseClass(MpdParsable):
         except mpd.MpdException, e:
             self.complete_to_musicdata_coords(lessonfile_ref, e)
             raise
-        track = mpd.score_to_percussion_tracks(score)[0]
+        track = mpd.score_to_tracks(score)[0]
         track.set_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.prepend_bpm(lessonfile_ref.get_tempo()[0],
                           lessonfile_ref.get_tempo()[1])
         return track
+    def play(self, lessonfile_ref, question):
+        soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
 
 
 class Rhythm(PercBaseClass):
-    def play(self, lessonfile_ref, question):
-        track = self._gen_track(lessonfile_ref, question)
-        track.replace_note(mpd.notename_to_int("c"), 37)
-        track.replace_note(mpd.notename_to_int("d"), 80)
-        soundcard.synth.play_track(track)
+    def get_mpd_music_string(self, lessonfile_ref):
+        v = list(mpd.parser.Lexer(self.m_musicdata))
+        for idx, (toc_type, toc) in enumerate(v):
+            if toc_type == mpd.parser.Lexer.NOTE:
+                if toc.m_pitch.semitone_pitch() == 48:
+                    toc.m_pitch.set_from_int(cfg.get_int("config/rhythm_perc"))
+                if toc.m_pitch.semitone_pitch() == 50:
+                    toc.m_pitch.set_from_int(cfg.get_int("config/countin_perc"))
+        return "\\rhythmstaff{\n%s\n}" % mpd.parser.Lexer.to_string(v)
 
 
 class Percussion(PercBaseClass):
-    def play(self, lessonfile_ref, question):
-        soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
+    pass
 
 class _MusicExternalPlayer(MusicBaseClass):
     def __init__(self, typeid, musicdata):
@@ -1292,6 +1272,15 @@ class QuestionsLessonfile(LessonfileCommon):
         header.random_transpose is set.
         """
         return " ".join(self.get_music_as_notename_list(varname))
+    def get_score(self, varname='music'):
+        """
+        Return the elems.Score object of the music in the variable named
+        by VARNAME. 
+        """
+        assert self._idx is not None
+        musicobj = self.m_questions[self._idx][varname]
+        assert isinstance(musicobj, MpdParsable)
+        return musicobj.get_score(self, as_name=varname)
     def has_question(self):
         """
         Return True if a question is selected.
@@ -1486,7 +1475,12 @@ class SingChordLessonfile(QuestionsLessonfile):
     pass
 
 
-class RhythmDictation2Lessonfile(LessonfileCommon):
+class RhythmDictation2Lessonfile(QuestionsLessonfile):
+    """
+    We inherit from QuestionsLessonfile just to get the .get_tempo()
+    method. Lots of the other methods from QuestionsLessonfile does
+    not make sense to use in the class.
+    """
     def parse_string(self, s, really_filename=None):
         LessonfileCommon.parse_string(self, s, really_filename)
         if not self.m_questions:
@@ -1508,6 +1502,7 @@ class RhythmDictation2Lessonfile(LessonfileCommon):
             for d in digits.split():
                 ret += Duration.new_from_string(d).get_rat_value()
             return ret
+        notename = mpd.MusicalPitch.new_from_int(cfg.get_int("config/rhythm_perc")).get_octave_notename()
         self._idx = random.randint(0, len(self.m_questions) - 1)
         # The score where the user enters his answer
         self.m_answer_score = score = elems.Score()
@@ -1535,14 +1530,14 @@ class RhythmDictation2Lessonfile(LessonfileCommon):
                 total += rat_len_of_digits(elem)
                 for e in elem.split(" "):
                     e = e.strip()
-                    n = elems.Note.new_from_string(u"c'%s" % e)
+                    n = elems.Note.new_from_string(u"%s%s" % (notename, e))
                     score.voice11.append(n)
                     count = 0
                 count += 1
             if count == max_count:
                 raise LessonfileException("Bad elements variable in the lessonfile. Too long elements.")
     def play_question(self):
-        tracks = mpd.score_to_percussion_tracks(self.m_question_score)
+        tracks = mpd.score_to_tracks(self.m_question_score)
         tracks[0].prepend_bpm(*self.m_questions[self._idx]['tempo'])
         soundcard.synth.play_track(*tracks)
 
